@@ -22,7 +22,6 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Dan Milstein <danmil@shore.net>                            *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 1497079 $                                          *
  ***************************************************************************/
 
 #ifndef JK_SERVICE_H
@@ -36,6 +35,7 @@
 #include "jk_msg_buff.h"
 
 #define JK_RETRIES 2
+#define JK_LB_RETRIES 2
 
 #ifdef __cplusplus
 extern "C"
@@ -65,7 +65,7 @@ struct jk_worker_env
 
     /* Virtual server handled - "*" is all virtual */
     char *virtual;
-    
+
     /* Optional APR pool used for configuration */
     void *pool;
 };
@@ -114,7 +114,7 @@ typedef struct svc_extension svc_extension_t;
  * container.  Contains the basic information about the request
  * (e.g. protocol, req_uri, etc), and also contains a series of methods
  * which provide access to core web server functionality (start_response,
- * read, write).  This class might be more accurately called ws_request.
+ * read, write).
  *
  * As with all the core jk classes, this is essentially an abstract base
  * class which is implemented/extended by classes which are specific to a
@@ -155,6 +155,11 @@ struct jk_ws_service
      */
     jk_pool_t *pool;
 
+    /*
+     * log context
+     */
+    jk_log_context_t *log_ctx;
+
     /*
      * CGI Environment needed by servlets
      */
@@ -168,6 +173,7 @@ struct jk_ws_service
     const char *auth_type;
     const char *query_string;
     const char *server_name;
+    const char *local_addr;
     unsigned server_port;
     char *server_software;
     jk_uint64_t content_length; /* 64 bit integer that represents the content */
@@ -180,6 +186,7 @@ struct jk_ws_service
      * SSL information
      *
      * is_ssl       - True if request is in ssl connection
+     * ssl_protocol-  Name of the SSL protocol (SSLv3, TLSv1, TLSv1.1, TLSv1.2)
      * ssl_cert     - If available, base64 ASN.1 encoded client certificates.
      * ssl_cert_len - Length of ssl_cert, 0 if certificates are not available.
      * ssl_cipher   - The ssl cipher suite in use.
@@ -189,6 +196,7 @@ struct jk_ws_service
      * case, we are passing NULL.
      */
     int is_ssl;
+    char *ssl_protocol;
     char *ssl_cert;
     unsigned ssl_cert_len;
     char *ssl_cipher;
@@ -427,7 +435,7 @@ struct jk_endpoint
      */
     int (JK_METHOD * service) (jk_endpoint_t *e,
                                jk_ws_service_t *s,
-                               jk_logger_t *l, int *is_error);
+                               jk_log_context_t *log_ctx, int *is_error);
 
     /*
      * Called when this particular endpoint has finished processing a
@@ -439,7 +447,7 @@ struct jk_endpoint
      * rather a pointer to a 'this' pointer.  This is necessary, because
      * we may need to free this object.
      */
-    int (JK_METHOD * done) (jk_endpoint_t **p, jk_logger_t *l);
+    int (JK_METHOD * done) (jk_endpoint_t **p, jk_log_context_t *log_ctx);
 };
 
 /*
@@ -508,14 +516,14 @@ struct jk_worker
      */
     int (JK_METHOD * validate) (jk_worker_t *w,
                                 jk_map_t *props,
-                                jk_worker_env_t *we, jk_logger_t *l);
+                                jk_worker_env_t *we, jk_log_context_t *log_ctx);
 
     /*
      * Update worker either from jk_status or reloading from workers.properties
      */
     int (JK_METHOD * update) (jk_worker_t *w,
                               jk_map_t *props,
-                              jk_worker_env_t *we, jk_logger_t *l);
+                              jk_worker_env_t *we, jk_log_context_t *log_ctx);
 
     /*
      * Do whatever initialization needs to be done to start this worker up.
@@ -523,7 +531,7 @@ struct jk_worker
      */
     int (JK_METHOD * init) (jk_worker_t *w,
                             jk_map_t *props,
-                            jk_worker_env_t *we, jk_logger_t *l);
+                            jk_worker_env_t *we, jk_log_context_t *log_ctx);
 
 
     /*
@@ -531,19 +539,24 @@ struct jk_worker
      * the endpoint is stored in pend.
      */
     int (JK_METHOD * get_endpoint) (jk_worker_t *w,
-                                    jk_endpoint_t **pend, jk_logger_t *l);
+                                    jk_endpoint_t **pend, jk_log_context_t *log_ctx);
 
     /*
      * Shutdown this worker.  The first argument is not a 'this' pointer,
      * but rather a pointer to 'this', so that the object can be free'd (I
      * think -- though that doesn't seem to be happening.  Hmmm).
      */
-    int (JK_METHOD * destroy) (jk_worker_t **w, jk_logger_t *l);
+    int (JK_METHOD * destroy) (jk_worker_t **w, jk_log_context_t *log_ctx);
 
     /*
      * Maintain this worker.
      */
-    int (JK_METHOD * maintain) (jk_worker_t *w, time_t now, jk_logger_t *l);
+    int (JK_METHOD * maintain) (jk_worker_t *w, time_t now, int global, jk_log_context_t *log_ctx);
+
+    /*
+     * Shut this worker down.
+     */
+    int (JK_METHOD * shutdown) (jk_worker_t *w, jk_log_context_t *log_ctx);
 
 };
 
@@ -562,7 +575,7 @@ struct jk_worker
  */
 typedef int (JK_METHOD * worker_factory) (jk_worker_t **w,
                                           const char *name,
-                                          jk_logger_t *l);
+                                          jk_log_context_t *log_ctx);
 
 void jk_init_ws_service(jk_ws_service_t *s);
 
